@@ -10,9 +10,10 @@ import numpy as np
 import PIL
 from PIL import PngImagePlugin
 from monai.config.type_definitions import KeysCollection
-from monai.transforms import MapTransform, Randomizable
+from monai.transforms import MapTransform, Randomizable, Transform
 from monai.utils.enums import WSIPatchKeys
 from monai.data.meta_tensor import MetaTensor
+from health_azure.logging import elapsed_timer
 from health_ml.utils.box_utils import Box
 from torchvision.transforms.functional import to_tensor
 
@@ -220,7 +221,7 @@ class Subsampled(MapTransform, Randomizable):
 
     def randomize(self, total_size: int) -> None:
         subsample_size = min(self.max_size, total_size)
-        self._indices = self.R.choice(total_size, size=subsample_size)
+        self._indices = self.R.choice(total_size, size=subsample_size, replace=False)
 
     def __call__(self, data: Mapping) -> Mapping:
         out_data = dict(data)  # create shallow copy
@@ -294,3 +295,20 @@ class MetaTensorToTensord(MapTransform):
             assert isinstance(out_data[key], MetaTensor), f"Expected MetaTensor, got {type(out_data[key])}"
             out_data[key] = out_data[key].as_tensor()
         return out_data
+
+
+class TimerWrapper(Transform):
+    """Transform that measures the time it takes to execute the transform. Useful for debugging. This can be used by
+    wrapping the transform in a TimerWrapperd transform.
+    Example:
+        transform = Compose([TimerWrapperd(LoadImaged(keys="image")), TimerWrapperd(MetaTensorToTensord(keys="image"))])
+    """
+
+    def __init__(self, transform: Callable) -> None:
+        self.transform = transform
+
+    def __call__(self, data: Mapping) -> Mapping:
+        message = f"{self.transform.__class__.__name__}, Slide {data[SlideKey.SLIDE_ID]}"
+        with elapsed_timer(message):
+            out_data = self.transform(data)
+        return out_data
