@@ -259,7 +259,8 @@ class DeepMILOutputsHandler:
     def __init__(self, outputs_root: Path, n_classes: int, tile_size: int, loading_params: LoadingParams,
                  class_names: Optional[Sequence[str]], primary_val_metric: MetricsKey,
                  maximise: bool, val_plot_options: Collection[PlotOption],
-                 test_plot_options: Collection[PlotOption], val_set_is_dist: bool = True) -> None:
+                 test_plot_options: Collection[PlotOption], val_set_is_dist: bool = True,
+                 save_intermediate_outputs: bool = True) -> None:
         """
         :param outputs_root: Root directory where to save all produced outputs.
         :param n_classes: Number of MIL classes (set `n_classes=1` for binary).
@@ -275,6 +276,7 @@ class DeepMILOutputsHandler:
             set is replicated on each process. This shouldn't affect the results, as we take the mean of the validation
             set metrics across processes. This is only relevant for the outputs_handler, which needs to know whether to
             gather the validation set outputs across processes or not before saving them.
+        :param save_intermediate_outputs: Whether to save intermediate outputs (e.g. after each epoch).
         """
         self.outputs_root = outputs_root
         self.n_classes = n_classes
@@ -282,7 +284,7 @@ class DeepMILOutputsHandler:
         self.outputs_policy = OutputsPolicy(outputs_root=outputs_root,
                                             primary_val_metric=primary_val_metric,
                                             maximise=maximise)
-
+        self.save_intermediate_outputs = save_intermediate_outputs
         self.tiles_selector: Optional[TilesSelector] = None
         self.val_plots_handler = DeepMILPlotsHandler(
             plot_options=val_plot_options,
@@ -367,7 +369,9 @@ class DeepMILOutputsHandler:
 
         # Only global rank-0 process should actually render and save the outputs
         # We also want to save the plots of the extra validation epoch
-        if self.outputs_policy.should_save_validation_outputs(metrics_dict, epoch, is_global_rank_zero, on_extra_val):
+        if self.save_intermediate_outputs and self.outputs_policy.should_save_validation_outputs(
+            metrics_dict, epoch, is_global_rank_zero, on_extra_val
+        ):
             # First move existing outputs to a temporary directory, to avoid mixing
             # outputs of different epochs in case writing fails halfway through
             if self.validation_outputs_dir.exists():
